import typer
from typing_extensions import Annotated

app = typer.Typer()

@app.command("encode")
def encode_mccmnc(mcc: Annotated[str, typer.Argument(..., help="3digits mcc")],
                  mnc: Annotated[str, typer.Argument(..., help="2 or 3 digits mnc")]) -> bytes:
    """
    Encode decimal MCC and MNC to a 3-byte hexadecimal PLMN
    Args:
        mcc: A 3-digit string, e.g., "405"
        mnc: A 2-digit or 3-digit string, e.g., "01" or "001"
    Returns:
        A 3-byte bytes object
    Example:
        "405", "01" -> 40 F5 10  (2-digit MNC)
        "405", "001" -> 40 50 10 (3-digit MNC)
    """
    if len(mcc) != 3:
        raise ValueError("MCC must be 3 digits")
    if len(mnc) not in (2, 3):
        raise ValueError("MNC must be 2 or 3 digits")
        
    # MCC 的每一位
    mcc1 = int(mcc[0])  # 第一位
    mcc2 = int(mcc[1])  # 第二位 
    mcc3 = int(mcc[2])  # 第三位
    
    # MNC 编码
    if len(mnc) == 2:
        mnc1 = int(mnc[0])  # 第一位
        mnc2 = int(mnc[1])  # 第二位
        mnc3 = 0x0F        # 2 位 MNC 用 F 填充
    else:  # 3 位 MNC
        mnc1 = int(mnc[0])  # 第一位
        mnc2 = int(mnc[1])  # 第二位
        mnc3 = int(mnc[2])  # 第三位
    
    # 组装 3 个字节，二进制移动
    byte1 = (mcc2 << 4) | mcc1  # mcc2 在高位|mcc1 在低位
    byte2 = (mnc3 << 4) | mcc3  # mnc3 在高位|mcc3 在低位
    byte3 = (mnc2 << 4) | mnc1  # mnc2 在高位|mnc1 在低位
    print(bytes([byte1, byte2, byte3]).hex())
    return bytes([byte1, byte2, byte3])

@app.command("decode")
def decode_plmn(buffer: Annotated[str, typer.Argument(..., help="3 bytes hex string of PLMN AVP value, stars without 0x, e.g. 40f101")]) -> tuple[str, str]:
    
    """
    Decode a 3-byte hexadecimal PLMN to decimal MCC and MNC
    Args:
        buffer: A string of 3-byte bytes object
    Returns:
        tuple[str, str]: (mcc, mnc)
    Example:
        40 F5 10 -> "405", "01"  (2-digit MNC)
        40 50 10 -> "405", "001" (3-digit MNC)
    """

    buffer = bytes.fromhex(buffer)
    if len(buffer) != 3:
        raise ValueError("Buffer must be 3 bytes")
    

    # 解析第一个字节
    mcc1 = buffer[0] & 0x0F         # 取低 4 位
    mcc2 = (buffer[0] >> 4) & 0x0F  # 取高 4 位
    
    # 解析第二个字节
    mcc3 = buffer[1] & 0x0F         # 取低 4 位
    mnc3 = (buffer[1] >> 4) & 0x0F  # 取高 4 位
    
    # 解析第三个字节 
    mnc1 = buffer[2] & 0x0F         # 取低 4 位
    mnc2 = (buffer[2] >> 4) & 0x0F  # 取高 4 位
    
    # 组装 MCC
    mcc = f"{mcc1}{mcc2}{mcc3}"
    
    # 组装 MNC (根据 mnc3 判断是 2 位还是 3 位)
    if mnc3 == 0x0F:
        mnc = f"{mnc1}{mnc2}"      # 2 位 MNC
    else:
        mnc = f"{mnc1}{mnc2}{mnc3}"  # 3 位 MNC
    
    print(f"{mcc},{mnc}")
    return mcc, mnc

if __name__ == "__main__":
    app()